
# q-expansion of newform 1127.2.a.d, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1127
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, -1], [1, 0], [2, -2], [0, 0], [2, -4], [1, 2], [0, 0], [6, -2], [-1, 0], [1, 4], [9, 0], [-2, 6], [1, -2], [0, -4], [1, -4], [10, -2], [4, 4], [-6, 12], [-6, 10], [-9, 2], [3, -6], [-6, 2], [-4, 4], [-4, 8], [0, 6], [6, -12], [12, -6], [10, -2], [-8, 12], [0, -4], [1, 6], [1, 8], [12, -6], [-5, -4], [14, -2], [-9, 10], [-8, 12], [13, -10], [-12, 16], [6, 4], [-15, 6], [10, 8], [-18, 10], [-3, -12], [-19, 4], [-6, 16], [-12, 0], [4, 12], [-4, -16], [-16, -2], [-9, 4], [-17, -2], [14, -12], [-12, -6], [-3, 2], [-22, 8], [3, -10], [12, -8], [13, -12], [0, -6], [-14, 8], [12, -6], [4, -12], [-7, -16], [-4, -2], [14, -28], [-3, -6], [-4, 22], [4, 16], [-5, -6], [3, 2], [6, 2], [12, 12], [26, 0], [12, 6], [12, -6], [0, 12], [17, -14], [-10, 16], [7, 18], [-24, 4], [-16, -12], [12, -10], [-14, 0], [-13, 28], [-1, 10], [18, -24], [12, -4], [27, 2], [0, 0], [14, 4], [12, -18], [9, -18], [13, 2], [-33, 6], [2, -22], [19, 10], [-4, 28], [4, -6], [21, 0], [37, -10], [-28, -2], [-4, -22], [-2, 10], [20, 6], [19, 6], [-11, 16], [-22, 20], [0, 4], [-21, 2], [16, -28], [-16, 10], [34, -2], [24, -38], [-4, -12], [12, 8], [14, -6], [21, 4], [27, 4], [8, 2], [38, -6], [-1, 0], [-22, 2], [1, 10], [-28, 24], [-24, -2], [12, -36], [-36, 0], [-2, 2], [-8, 8], [5, -6], [-28, -4], [-16, -12], [-14, 24], [23, -14], [42, -2], [-32, 14], [-12, -20], [38, 2], [-18, 24], [33, -20], [30, 12], [7, -30], [0, 28], [14, 0], [-4, 26], [-30, 24], [9, 30], [-19, -12], [1, -2], [-42, 0], [-12, 22], [0, -16], [-7, 16], [-6, -30], [-2, 32], [0, -6], [-43, 14], [46, -18], [-32, 34], [-9, 22], [22, 16], [-15, 26], [-30, 6], [-10, 14], [-2, -10], [-44, 8], [10, -28], [42, -30], [29, -2], [-10, -10], [-9, 18], [12, -12], [48, -2], [23, -8], [0, -18], [-15, -2], [-9, -12], [-38, 6], [-36, 24], [-7, 6], [44, -8], [21, -20], [14, -20], [-34, 40], [-32, 8], [-6, 20], [42, -32], [29, 4], [-4, -16], [17, -30], [-16, -16], [-54, 2], [30, -32], [10, -22], [-12, -12], [22, -22], [-8, 30], [20, -22], [-33, 44], [-12, -14], [24, -24], [7, -10], [-30, 4], [-4, 14], [43, -14], [-6, -18], [-21, 18], [27, -24], [-29, 32], [-29, 24], [-59, -2], [-12, 30], [50, -10], [-12, 24], [-36, 2], [3, -18], [11, 14], [2, 26], [-21, 0], [3, 32], [-6, -18], [9, -48], [-20, 6], [-3, 12], [15, 24], [6, -24], [5, 26], [-30, 6], [-24, -24], [62, -20], [64, 2], [10, -40], [22, 8], [12, -22], [-44, 26], [-24, 0], [-6, -16], [-33, 38], [3, -12], [-12, -14], [21, -20], [21, 0], [-33, 8], [46, 4], [-11, 32], [-59, 8], [-22, 40], [-22, -18], [-30, 4], [0, 38], [46, -28], [22, 2], [0, -30], [6, 0], [21, -30], [14, 22], [14, 0], [-37, 38], [31, 6], [40, 6], [56, -10], [50, -20], [48, 0], [18, -36], [46, -12], [-42, 24], [-40, -10], [14, -50], [-3, -36], [19, -30], [10, 20], [4, 2], [-28, 10], [-5, 38], [-3, 4], [-16, -32], [-6, -24], [-18, -36], [-8, -16], [-57, 16], [-50, -6], [-14, -14], [2, -12], [52, -12], [-24, 8], [10, 30], [54, -28], [32, -24], [-2, 4], [66, -16], [-32, 26], [-18, 0], [22, -20], [13, 12], [30, 28], [24, -16], [-15, 58], [-1, 32], [15, -48], [28, -64], [-30, 24], [12, -6], [-43, -2], [-16, 10], [15, 8], [-12, 48], [24, 30], [58, -4], [-18, -24], [-31, 44], [-42, -18], [-4, 4], [49, -46], [26, 4], [0, 36], [7, -4], [-37, 22], [24, 16], [7, -56], [-32, -18], [-28, 30], [17, 2], [-72, 12], [34, 4], [48, 24], [-30, 0], [35, -60], [54, -24], [12, -44], [49, -60], [-33, 68], [-17, 24], [19, -32], [-6, 24], [15, -28], [-58, -18], [56, -4], [29, -38], [-20, 24], [27, 12], [-32, 4], [30, -44], [-54, -12], [74, -10], [-36, 20], [-55, -6], [20, -24], [-42, 0], [-54, 36], [-27, 0], [5, 24], [-70, 16], [-6, 48], [36, -4], [14, -16], [48, -12], [-6, 26], [9, 8], [-12, 12], [2, -50], [-25, -26], [53, -62], [-18, 64], [-69, 12], [-59, -16], [61, -38], [26, -32], [-30, -30], [73, -38], [12, -24], [11, 34], [-8, 32], [0, -34], [-29, 32], [-1, 32], [38, -2], [-6, 56], [-40, -14], [25, -54], [30, -46], [6, 28], [20, 36], [-9, -36], [-40, 30], [3, -48], [1, -58], [-6, 48], [54, -34], [-4, 14], [24, 12], [-42, 12], [47, -62], [-24, -4], [36, -36], [34, -72], [44, 12], [44, -14], [4, -18], [38, -64], [-7, -2], [-35, 16], [12, -8], [-12, 18], [-83, -10], [-7, 24], [46, -4], [13, -16], [40, 26], [-64, -16], [81, -30], [-1, -28], [-22, -22], [26, -36], [-54, -18], [-53, 10], [-16, 28], [-12, -24], [12, 30], [-60, 6], [12, -30], [-12, -42], [48, 8], [24, 0], [-21, 34], [-19, -22], [6, -60], [26, -32], [-3, -12], [-47, 76]]
